@@ -1,19 +1,24 @@
+# Created on 2019-06-26 by kevincar <kevincarrolldavis@gmail.com>
 """
 BLE Client for CoreBluetooth on macOS
-
-Created on 2019-06-26 by kevincar <kevincarrolldavis@gmail.com>
 """
 
+import sys
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    if sys.platform != "darwin":
+        assert False, "This backend is only available on macOS"
+
 import asyncio
 import logging
-import sys
-import uuid
-from typing import Optional, Set, Union
+from typing import Any, Optional, Union
 
 if sys.version_info < (3, 12):
-    from typing_extensions import Buffer
+    from typing_extensions import Buffer, override
 else:
     from collections.abc import Buffer
+    from typing import override
 
 from CoreBluetooth import (
     CBUUID,
@@ -24,23 +29,19 @@ from CoreBluetooth import (
 )
 from Foundation import NSArray, NSData
 
-from ... import BleakScanner
-from ...exc import (
-    BleakCharacteristicNotFoundError,
-    BleakDeviceNotFoundError,
-    BleakError,
-)
-from ..characteristic import BleakGATTCharacteristic
-from ..client import BaseBleakClient, NotifyCallback
-from ..device import BLEDevice
-from ..service import BleakGATTServiceCollection
-from .CentralManagerDelegate import CentralManagerDelegate
-from .characteristic import BleakGATTCharacteristicCoreBluetooth
-from .descriptor import BleakGATTDescriptorCoreBluetooth
-from .PeripheralDelegate import PeripheralDelegate
-from .scanner import BleakScannerCoreBluetooth
-from .service import BleakGATTServiceCoreBluetooth
-from .utils import cb_uuid_to_str
+from bleak import BleakScanner
+from bleak.args.corebluetooth import CBStartNotifyArgs
+from bleak.assigned_numbers import gatt_char_props_to_strs
+from bleak.backends.characteristic import BleakGATTCharacteristic
+from bleak.backends.client import BaseBleakClient, NotifyCallback
+from bleak.backends.corebluetooth.CentralManagerDelegate import CentralManagerDelegate
+from bleak.backends.corebluetooth.PeripheralDelegate import PeripheralDelegate
+from bleak.backends.corebluetooth.scanner import BleakScannerCoreBluetooth
+from bleak.backends.corebluetooth.utils import cb_uuid_to_str
+from bleak.backends.descriptor import BleakGATTDescriptor
+from bleak.backends.device import BLEDevice
+from bleak.backends.service import BleakGATTService, BleakGATTServiceCollection
+from bleak.exc import BleakDeviceNotFoundError, BleakError
 
 logger = logging.getLogger(__name__)
 
@@ -60,8 +61,8 @@ class BleakClientCoreBluetooth(BaseBleakClient):
     def __init__(
         self,
         address_or_ble_device: Union[BLEDevice, str],
-        services: Optional[Set[str]] = None,
-        **kwargs,
+        services: Optional[set[str]] = None,
+        **kwargs: Any,
     ):
         super(BleakClientCoreBluetooth, self).__init__(address_or_ble_device, **kwargs)
 
@@ -76,7 +77,9 @@ class BleakClientCoreBluetooth(BaseBleakClient):
             ) = address_or_ble_device.details
 
         self._requested_services = (
-            NSArray.alloc().initWithArray_(list(map(CBUUID.UUIDWithString_, services)))
+            NSArray[CBUUID]
+            .alloc()
+            .initWithArray_(list(map(CBUUID.UUIDWithString_, services)))
             if services
             else None
         )
@@ -84,16 +87,16 @@ class BleakClientCoreBluetooth(BaseBleakClient):
     def __str__(self) -> str:
         return "BleakClientCoreBluetooth ({})".format(self.address)
 
-    async def connect(self, **kwargs) -> bool:
+    @override
+    async def connect(self, pair: bool, **kwargs: Any) -> None:
         """Connect to a specified Peripheral
 
         Keyword Args:
             timeout (float): Timeout for required ``BleakScanner.find_device_by_address`` call. Defaults to 10.0.
-
-        Returns:
-            Boolean representing connection status.
-
         """
+        if pair:
+            logger.debug("Explicit pairing is not available in CoreBluetooth.")
+
         timeout = kwargs.get("timeout", self._timeout)
         if self._peripheral is None:
             device = await BleakScanner.find_device_by_address(
@@ -136,37 +139,38 @@ class BleakClientCoreBluetooth(BaseBleakClient):
         await manager.connect(self._peripheral, disconnect_callback, timeout=timeout)
 
         # Now get services
-        await self.get_services()
+        await self._get_services()
 
-        return True
-
-    async def disconnect(self) -> bool:
+    @override
+    async def disconnect(self) -> None:
         """Disconnect from the peripheral device"""
         if (
             self._peripheral is None
             or self._peripheral.state() != CBPeripheralStateConnected
         ):
-            return True
+            return
 
+        assert self._central_manager_delegate
         await self._central_manager_delegate.disconnect(self._peripheral)
 
-        return True
-
     @property
+    @override
     def is_connected(self) -> bool:
         """Checks for current active connection"""
-        return self._DeprecatedIsConnectedReturn(
+        return (
             False
             if self._peripheral is None
             else self._peripheral.state() == CBPeripheralStateConnected
         )
 
     @property
+    @override
     def mtu_size(self) -> int:
         """Get ATT MTU size for active connection"""
         # Use type CBCharacteristicWriteWithoutResponse to get maximum write
         # value length based on the negotiated ATT MTU size. Add the ATT header
         # length (+3) to get the actual ATT MTU size.
+        assert self._peripheral
         return (
             self._peripheral.maximumWriteValueLengthForType_(
                 CBCharacteristicWriteWithoutResponse
@@ -174,36 +178,37 @@ class BleakClientCoreBluetooth(BaseBleakClient):
             + 3
         )
 
-    async def pair(self, *args, **kwargs) -> bool:
+    @override
+    async def pair(self, *args: Any, **kwargs: Any) -> None:
         """Attempt to pair with a peripheral.
 
-        .. note::
-
-            This is not available on macOS since there is not explicit method to do a pairing, Instead the docs
-            state that it "auto-pairs" when trying to read a characteristic that requires encryption, something
-            Bleak cannot do apparently.
+        Raises:
+            NotImplementedError:
+                This is not available on macOS since there is not explicit API
+                to do a pairing. Instead, the docs state that it "auto-pairs",
+                when trying to read a characteristic that requires encryption.
 
         Reference:
 
             - `Apple Docs <https://developer.apple.com/library/archive/documentation/NetworkingInternetWeb/Conceptual/CoreBluetooth_concepts/BestPracticesForSettingUpYourIOSDeviceAsAPeripheral/BestPracticesForSettingUpYourIOSDeviceAsAPeripheral.html#//apple_ref/doc/uid/TP40013257-CH5-SW1>`_
             - `Stack Overflow post #1 <https://stackoverflow.com/questions/25254932/can-you-pair-a-bluetooth-le-device-in-an-ios-app>`_
             - `Stack Overflow post #2 <https://stackoverflow.com/questions/47546690/ios-bluetooth-pairing-request-dialog-can-i-know-the-users-choice>`_
-
-        Returns:
-            Boolean regarding success of pairing.
-
         """
         raise NotImplementedError("Pairing is not available in Core Bluetooth.")
 
-    async def unpair(self) -> bool:
+    @override
+    async def unpair(self) -> None:
         """
+        Remove pairing information for a peripheral.
 
-        Returns:
-
+        Raises:
+            NotImplementedError:
+                This is not available on macOS since there is not explicit API
+                to do a pairing.
         """
         raise NotImplementedError("Pairing is not available in Core Bluetooth.")
 
-    async def get_services(self, **kwargs) -> BleakGATTServiceCollection:
+    async def _get_services(self) -> BleakGATTServiceCollection:
         """Get all services registered for this GATT server.
 
         Returns:
@@ -216,79 +221,77 @@ class BleakClientCoreBluetooth(BaseBleakClient):
         services = BleakGATTServiceCollection()
 
         logger.debug("Retrieving services...")
+        assert self._delegate
         cb_services = await self._delegate.discover_services(self._requested_services)
 
         for service in cb_services:
+            serv = BleakGATTService(
+                service, service.startHandle(), cb_uuid_to_str(service.UUID())
+            )
+            services.add_service(serv)
+
             serviceUUID = service.UUID().UUIDString()
             logger.debug(
                 "Retrieving characteristics for service {}".format(serviceUUID)
             )
             characteristics = await self._delegate.discover_characteristics(service)
 
-            services.add_service(BleakGATTServiceCoreBluetooth(service))
-
             for characteristic in characteristics:
                 cUUID = characteristic.UUID().UUIDString()
                 logger.debug(
                     "Retrieving descriptors for characteristic {}".format(cUUID)
                 )
-                descriptors = await self._delegate.discover_descriptors(characteristic)
 
-                services.add_characteristic(
-                    BleakGATTCharacteristicCoreBluetooth(
-                        characteristic,
-                        lambda: self._peripheral.maximumWriteValueLengthForType_(
-                            CBCharacteristicWriteWithoutResponse
-                        ),
-                    )
+                char = BleakGATTCharacteristic(
+                    characteristic,
+                    characteristic.handle(),
+                    cb_uuid_to_str(characteristic.UUID()),
+                    list(gatt_char_props_to_strs(characteristic.properties())),
+                    lambda: self._peripheral.maximumWriteValueLengthForType_(
+                        CBCharacteristicWriteWithoutResponse
+                    ),
+                    serv,
                 )
+                services.add_characteristic(char)
+
+                descriptors = await self._delegate.discover_descriptors(characteristic)
                 for descriptor in descriptors:
-                    services.add_descriptor(
-                        BleakGATTDescriptorCoreBluetooth(
-                            descriptor,
-                            cb_uuid_to_str(characteristic.UUID()),
-                            int(characteristic.handle()),
-                        )
+                    desc = BleakGATTDescriptor(
+                        descriptor,
+                        int(descriptor.handle()),
+                        cb_uuid_to_str(descriptor.UUID()),
+                        char,
                     )
+                    services.add_descriptor(desc)
+
         logger.debug("Services resolved for %s", str(self))
         self.services = services
         return self.services
 
+    @override
     async def read_gatt_char(
-        self,
-        char_specifier: Union[BleakGATTCharacteristic, int, str, uuid.UUID],
-        use_cached: bool = False,
-        **kwargs,
+        self, characteristic: BleakGATTCharacteristic, **kwargs: Any
     ) -> bytearray:
         """Perform read operation on the specified GATT characteristic.
 
         Args:
-            char_specifier (BleakGATTCharacteristic, int, str or UUID): The characteristic to read from,
-                specified by either integer handle, UUID or directly by the
-                BleakGATTCharacteristic object representing it.
-            use_cached (bool): `False` forces macOS to read the value from the
-                device again and not use its own cached value. Defaults to `False`.
+            characteristic (BleakGATTCharacteristic): The characteristic to read from.
 
         Returns:
             (bytearray) The read data.
 
         """
-        if not isinstance(char_specifier, BleakGATTCharacteristic):
-            characteristic = self.services.get_characteristic(char_specifier)
-        else:
-            characteristic = char_specifier
-        if not characteristic:
-            raise BleakCharacteristicNotFoundError(char_specifier)
-
+        assert self._delegate
         output = await self._delegate.read_characteristic(
-            characteristic.obj, use_cached=use_cached
+            characteristic.obj, use_cached=kwargs.get("use_cached", False)
         )
         value = bytearray(output)
         logger.debug("Read Characteristic {0} : {1}".format(characteristic.uuid, value))
         return value
 
+    @override
     async def read_gatt_descriptor(
-        self, handle: int, use_cached: bool = False, **kwargs
+        self, descriptor: BleakGATTDescriptor, **kwargs: Any
     ) -> bytearray:
         """Perform read operation on the specified GATT descriptor.
 
@@ -300,12 +303,9 @@ class BleakClientCoreBluetooth(BaseBleakClient):
         Returns:
             (bytearray) The read data.
         """
-        descriptor = self.services.get_descriptor(handle)
-        if not descriptor:
-            raise BleakError("Descriptor {} was not found!".format(handle))
-
+        assert self._delegate
         output = await self._delegate.read_descriptor(
-            descriptor.obj, use_cached=use_cached
+            descriptor.obj, use_cached=kwargs.get("use_cached", False)
         )
         if isinstance(
             output, str
@@ -313,14 +313,12 @@ class BleakClientCoreBluetooth(BaseBleakClient):
             value = bytearray(output.encode("utf-8"))
         else:  # _NSInlineData
             value = bytearray(output)  # value.getBytes_length_(None, len(value))
-        logger.debug("Read Descriptor {0} : {1}".format(handle, value))
+        logger.debug("Read Descriptor %d : %r", descriptor.handle, value)
         return value
 
+    @override
     async def write_gatt_char(
-        self,
-        characteristic: BleakGATTCharacteristic,
-        data: Buffer,
-        response: bool,
+        self, characteristic: BleakGATTCharacteristic, data: Buffer, response: bool
     ) -> None:
         value = NSData.alloc().initWithBytes_length_(data, len(data))
         await self._delegate.write_characteristic(
@@ -334,56 +332,54 @@ class BleakClientCoreBluetooth(BaseBleakClient):
         )
         logger.debug(f"Write Characteristic {characteristic.uuid} : {data}")
 
-    async def write_gatt_descriptor(self, handle: int, data: Buffer) -> None:
+    @override
+    async def write_gatt_descriptor(
+        self, descriptor: BleakGATTDescriptor, data: Buffer
+    ) -> None:
         """Perform a write operation on the specified GATT descriptor.
 
         Args:
-            handle: The handle of the descriptor to read from.
+            descriptor: The descriptor to read from.
             data: The data to send (any bytes-like object).
 
         """
-        descriptor = self.services.get_descriptor(handle)
-        if not descriptor:
-            raise BleakError("Descriptor {} was not found!".format(handle))
-
+        assert self._delegate
         value = NSData.alloc().initWithBytes_length_(data, len(data))
         await self._delegate.write_descriptor(descriptor.obj, value)
-        logger.debug("Write Descriptor {0} : {1}".format(handle, data))
+        logger.debug("Write Descriptor %d : %r", descriptor.handle, data)
 
+    @override
     async def start_notify(
         self,
         characteristic: BleakGATTCharacteristic,
         callback: NotifyCallback,
-        **kwargs,
+        *,
+        cb: CBStartNotifyArgs,
+        **kwargs: Any,
     ) -> None:
         """
         Activate notifications/indications on a characteristic.
         """
         assert self._delegate is not None
 
-        await self._delegate.start_notifications(characteristic.obj, callback)
+        await self._delegate.start_notifications(
+            characteristic.obj,
+            callback,
+            cb.get("notification_discriminator"),
+        )
 
-    async def stop_notify(
-        self, char_specifier: Union[BleakGATTCharacteristic, int, str, uuid.UUID]
-    ) -> None:
+    @override
+    async def stop_notify(self, characteristic: BleakGATTCharacteristic) -> None:
         """Deactivate notification/indication on a specified characteristic.
 
         Args:
-            char_specifier (BleakGATTCharacteristic, int, str or UUID): The characteristic to deactivate
-                notification/indication on, specified by either integer handle, UUID or
-                directly by the BleakGATTCharacteristic object representing it.
-
-
+            characteristic (BleakGATTCharacteristic: The characteristic to deactivate
+                notification/indication on.
         """
-        if not isinstance(char_specifier, BleakGATTCharacteristic):
-            characteristic = self.services.get_characteristic(char_specifier)
-        else:
-            characteristic = char_specifier
-        if not characteristic:
-            raise BleakCharacteristicNotFoundError(char_specifier)
-
+        assert self._delegate
         await self._delegate.stop_notifications(characteristic.obj)
 
     async def get_rssi(self) -> int:
         """To get RSSI value in dBm of the connected Peripheral"""
+        assert self._delegate
         return int(await self._delegate.read_rssi())
