@@ -12,7 +12,7 @@ CHAR_UUID = "00000000-0000-0000-0000-000000000000"
 
 
 async def main():
-    queue = asyncio.Queue()
+    queue = asyncio.Queue[BLEDevice]()
 
     def callback(device: BLEDevice, adv: AdvertisementData) -> None:
         # can use advertising data to filter here
@@ -26,8 +26,8 @@ async def main():
         # BlueZ doesn't have a proper way to get the MTU, so we have this hack.
         # If this doesn't work for you, you can set the client._mtu_size attribute
         # to override the value instead.
-        if client._backend.__class__.__name__ == "BleakClientBlueZDBus":
-            await client._backend._acquire_mtu()
+        if client._backend.__class__.__name__ == "BleakClientBlueZDBus":  # type: ignore
+            await client._backend._acquire_mtu()  # type: ignore
 
         print("MTU:", client.mtu_size)
 
